"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("./util");
var Printer_1 = require("./Printer");
var WellKnown_1 = require("./WellKnown");
var FieldTypes_1 = require("./FieldTypes");
function printFileDescriptorTSGRPC(fileDescriptor, exportMap) {
    if (fileDescriptor.getServiceList().length === 0) {
        return "";
    }
    var fileName = fileDescriptor.getName();
    var packageName = fileDescriptor.getPackage();
    var upToRoot = util_1.getPathToRoot(fileName);
    var printer = new Printer_1.Printer(0);
    printer.printLn("// package: " + packageName);
    printer.printLn("// file: " + fileDescriptor.getName());
    printer.printEmptyLn();
    var asPseudoNamespace = util_1.filePathToPseudoNamespace(fileName);
    printer.printLn("import * as GRPC from \"grpc\";");
    printer.printLn("import * as " + asPseudoNamespace + " from \"" + upToRoot + util_1.replaceProtoSuffix(fileName) + "\";");
    fileDescriptor.getDependencyList()
        .filter(function (dependency) {
        return isUsed(fileDescriptor, util_1.filePathToPseudoNamespace(dependency), exportMap);
    })
        .forEach(function (dependency) {
        var pseudoNamespace = util_1.filePathToPseudoNamespace(dependency);
        if (dependency in WellKnown_1.WellKnownTypesMap) {
            printer.printLn("import * as " + pseudoNamespace + " from \"" + WellKnown_1.WellKnownTypesMap[dependency] + "\";");
        }
        else {
            var filePath = util_1.replaceProtoSuffix(dependency);
            printer.printLn("import * as " + pseudoNamespace + " from \"" + (upToRoot + filePath) + "\";");
        }
    });
    fileDescriptor.getServiceList().forEach(function (service) {
        printer.printLn("");
        printer.printLn("export = " + service.getName() + ";");
        printer.printLn("declare namespace " + service.getName() + " {");
        var methodImplTypesPrinter = new Printer_1.Printer(1);
        methodImplTypesPrinter.printLn("namespace MethodImplTypes {");
        var methodImplsPrinter = new Printer_1.Printer(1);
        methodImplsPrinter.printLn("interface MethodImpls {");
        var clientClassName = service.getName() + "ClientClass";
        var clientClassPrinter = new Printer_1.Printer(1);
        clientClassPrinter.printLn("class " + clientClassName + " extends GRPC.Client {");
        var serviceInterfaceName = "I" + service.getName() + "Service";
        var servicePrinter = new Printer_1.Printer(1);
        servicePrinter.printLn("interface " + serviceInterfaceName + " extends GRPC.ServiceDefinition<MethodImpls> {");
        service.getMethodList().forEach(function (method) {
            var isClientStreaming = method.getClientStreaming();
            var isServerStreaming = method.getServerStreaming();
            var requestMessageTypeName = FieldTypes_1.getFieldType(FieldTypes_1.MESSAGE_TYPE, method.getInputType().slice(1), "", exportMap);
            var responseMessageTypeName = FieldTypes_1.getFieldType(FieldTypes_1.MESSAGE_TYPE, method.getOutputType().slice(1), "", exportMap);
            var grpcMethodName = lowerFirstLetter(method.getName());
            if (!isClientStreaming && !isServerStreaming) {
                methodImplTypesPrinter.printIndentedLn("type " + grpcMethodName + " = GRPC.handleUnaryCall<" + requestMessageTypeName + ", " + responseMessageTypeName + ">;");
                methodImplsPrinter.printIndentedLn(grpcMethodName + ": MethodImplTypes." + grpcMethodName + ";");
                clientClassPrinter.printIndentedLn(grpcMethodName + "(" +
                    ("request: " + requestMessageTypeName + ", ") +
                    "options: Partial<GRPC.CallOptions>, " +
                    ("callback: GRPC.requestCallback<" + responseMessageTypeName + ">") +
                    "): GRPC.ClientUnaryCall;");
                clientClassPrinter.printIndentedLn(grpcMethodName + "(" +
                    ("request: " + requestMessageTypeName + ", ") +
                    ("callback: GRPC.requestCallback<" + responseMessageTypeName + ">") +
                    "): GRPC.ClientUnaryCall;");
            }
            else if (!isClientStreaming && isServerStreaming) {
                methodImplTypesPrinter.printIndentedLn("type " + grpcMethodName + " = GRPC.handleServerStreamingCall<" + requestMessageTypeName + ", " + responseMessageTypeName + ">;");
                methodImplsPrinter.printIndentedLn(grpcMethodName + ": MethodImplTypes." + grpcMethodName + ";");
                clientClassPrinter.printIndentedLn(grpcMethodName + "(" +
                    ("request: " + requestMessageTypeName) +
                    ("): GRPC.ClientReadableStream<" + responseMessageTypeName + ">;"));
                clientClassPrinter.printIndentedLn(grpcMethodName + "(" +
                    ("request: " + requestMessageTypeName + ", ") +
                    "options: Partial<GRPC.CallOptions>" +
                    ("): GRPC.ClientReadableStream<" + responseMessageTypeName + ">;"));
            }
            else if (isClientStreaming && !isServerStreaming) {
                methodImplTypesPrinter.printIndentedLn("type " + grpcMethodName + " = GRPC.handleClientStreamingCall<" + requestMessageTypeName + ", " + responseMessageTypeName + ">;");
                methodImplsPrinter.printIndentedLn(grpcMethodName + ": MethodImplTypes." + grpcMethodName + ";");
                clientClassPrinter.printIndentedLn(grpcMethodName + "(" +
                    ("callback: GRPC.requestCallback<" + responseMessageTypeName + ">") +
                    ("): GRPC.ClientWritableStream<" + requestMessageTypeName + ">;"));
                clientClassPrinter.printIndentedLn(grpcMethodName + "(" +
                    "options: Partial<GRPC.CallOptions>, " +
                    ("callback: GRPC.requestCallback<" + responseMessageTypeName + ">") +
                    ("): GRPC.ClientWritableStream<" + requestMessageTypeName + ">;"));
            }
            else {
                methodImplTypesPrinter.printIndentedLn("type " + grpcMethodName + " = GRPC.handleBidiStreamingCall<" + requestMessageTypeName + ", " + responseMessageTypeName + ">;");
                methodImplsPrinter.printIndentedLn(grpcMethodName + ": MethodImplTypes." + grpcMethodName + ";");
                clientClassPrinter.printIndentedLn(grpcMethodName + "(" +
                    "options: Partial<GRPC.CallOptions>" +
                    ("): GRPC.ClientDuplexStream<" + requestMessageTypeName + ", " + responseMessageTypeName + ">;"));
                clientClassPrinter.printIndentedLn(grpcMethodName + "(): GRPC.ClientDuplexStream<" + requestMessageTypeName + ", " + responseMessageTypeName + ">;");
            }
            servicePrinter.printIndentedLn(grpcMethodName + ": GRPC.MethodDefinition<" + requestMessageTypeName + ", " + responseMessageTypeName + ">;");
        });
        methodImplTypesPrinter.printLn("}");
        methodImplsPrinter.printLn("}");
        clientClassPrinter.printLn("}");
        servicePrinter.printLn("}");
        printer.print(methodImplTypesPrinter.output);
        printer.print(methodImplsPrinter.output);
        printer.printLn("");
        printer.print(clientClassPrinter.output);
        printer.print(servicePrinter.output);
        printer.printLn("");
        printer.printIndentedLn("const " + service.getName() + "Service: " + serviceInterfaceName + ";");
        printer.printIndentedLn("const " + service.getName() + "Client: typeof " + clientClassName + ";");
        printer.printLn("}");
    });
    return printer.getOutput();
}
exports.printFileDescriptorTSGRPC = printFileDescriptorTSGRPC;
function isUsed(fileDescriptor, pseudoNamespace, exportMap) {
    return fileDescriptor.getServiceList().some(function (service) {
        return service.getMethodList().some(function (method) {
            var requestMessageTypeName = FieldTypes_1.getFieldType(FieldTypes_1.MESSAGE_TYPE, method.getInputType().slice(1), "", exportMap);
            var responseMessageTypeName = FieldTypes_1.getFieldType(FieldTypes_1.MESSAGE_TYPE, method.getOutputType().slice(1), "", exportMap);
            var namespacePackage = pseudoNamespace + ".";
            return (requestMessageTypeName.indexOf(namespacePackage) === 0 ||
                responseMessageTypeName.indexOf(namespacePackage) === 0);
        });
    });
}
function lowerFirstLetter(str) {
    return str.charAt(0).toLowerCase() + str.slice(1);
}
//# sourceMappingURL=grpc.js.map